@@ -246,7 +246,20 @@ def verify_config(verbose=True):
     ok = True
     config = get_config()
     if not config.ready:
-        config.load()
+        try:
+            config.load()
+        except ValueError as e:
+            config_key = getattr(e, "dallinger_config_key", None)
+            if config_key is not None:
+                message = "Configuration for {} is invalid: ".format(config_key)
+            else:
+                message = "Configuration is invalid: "
+            log("✗ " + message + str(e), delay=0, chevrons=False, verbose=verbose)
+
+            config_value = getattr(e, "dallinger_config_value", None)
+            if verbose and config_value:
+                log("  Value supplied was " + config_value, chevrons=False)
+            return False
     # Check base_payment is correct
     try:
         base_pay = config.get("base_payment")
@@ -333,12 +346,16 @@ def require_exp_directory(f):
     """Decorator to verify that a command is run inside a valid Dallinger
     experiment directory.
     """
-    error = "The current directory is not a valid Dallinger experiment."
+    error_one = "The current directory is not a valid Dallinger experiment."
+    error_two = "There are problems with the current experiment. Please check with dallinger verify."
 
     @wraps(f)
     def wrapper(**kwargs):
-        if not verify_directory(kwargs.get("verbose")):
-            raise click.UsageError(error)
+        try:
+            if not verify_package(kwargs.get("verbose")):
+                raise click.UsageError(error_one)
+        except ValueError:
+            raise click.UsageError(error_two)
         return f(**kwargs)
 
     return wrapper
@@ -745,6 +762,48 @@ def expire(app, sandbox, exit=True):
         sys.exit(1)
 
 
+@dallinger.command()
+@click.option("--hit_id", required=True)
+@click.option("--assignments", required=True, type=int)
+@click.option("--duration_hours", type=float)
+@click.option(
+    "--sandbox", is_flag=True, flag_value=True, help="HIT is in the MTurk sandbox"
+)
+def extend_mturk_hit(hit_id, assignments, duration_hours, sandbox):
+    """Add assignments, and optionally time, to an existing MTurk HIT."""
+    out = Output()
+    config = get_config()
+    config.load()
+    mode = "sandbox" if sandbox else "live"
+    assignments_presented = "assignments" if assignments > 1 else "assignment"
+    duration_presented = duration_hours or 0.0
+
+    with config.override({"mode": mode}):
+        if not click.confirm(
+            "\n\nYou are about to add {:.1f} hours and {} {} to {} HIT {}.\n"
+            "Continue?".format(
+                duration_presented, assignments, assignments_presented, mode, hit_id
+            )
+        ):
+            out.log("Aborting...")
+            return
+
+        service = _mturk_service_from_config(sandbox)
+        try:
+            hit_info = service.extend_hit(
+                hit_id=hit_id, number=assignments, duration_hours=duration_hours
+            )
+        except MTurkServiceException as ex:
+            out.error(
+                "HIT extension failed with the following error:\n{}".format(ex),
+                delay=0,
+            )
+            return
+
+    out.log("Updated HIT Details", delay=0)
+    out.log(tabulate.tabulate(hit_info.items()), chevrons=False, delay=0)
+
+
 @dallinger.command()
 @click.option("--app", default=None, callback=verify_id, help="Experiment id")
 @click.confirmation_option(prompt="Are you sure you want to destroy the app?")
