@@ -1,5 +1,7 @@
 # -*- coding: utf-8 -*-
 from __future__ import unicode_literals
+import codecs
+import json
 import os
 import pkg_resources
 import re
@@ -12,7 +14,10 @@ import tempfile
 import threading
 import time
 import webbrowser
+
 from six.moves import shlex_quote as quote
+from six.moves.urllib.parse import urlparse
+from six.moves.urllib.parse import urlunparse
 from unicodedata import normalize
 
 from dallinger import data
@@ -28,8 +33,10 @@ from dallinger.utils import dallinger_package_path
 from dallinger.utils import ensure_directory
 from dallinger.utils import get_base_url
 from dallinger.utils import GitClient
+from faker import Faker
 
-config = get_config()
+
+fake = Faker()
 
 
 def _make_chrome(path):
@@ -64,6 +71,7 @@ def new_webbrowser_profile():
         ]
         return new_firefox
     elif sys.platform == "darwin":
+        config = get_config()
         chrome_path = config.get("chrome-path")
         if os.path.exists(chrome_path):
             return _make_chrome(chrome_path)
@@ -162,6 +170,70 @@ class ExperimentFileSource(object):
                 yield legit
 
 
+class ExplicitFileSource(object):
+    """Add files that are explicitly requested by the experimenter with a hook function.
+    """
+
+    def __init__(self, root_dir="."):
+        self.root = root_dir
+
+    def _get_mapping(self, dst):
+        from dallinger.config import initialize_experiment_package
+
+        initialize_experiment_package(dst)
+        from dallinger.experiment import load
+
+        exp_class = load()
+        extra_files = getattr(exp_class, "extra_files", None)
+        if extra_files is None:
+            try:
+                from dallinger_experiment.experiment import extra_files
+            except ImportError:
+                try:
+                    from dallinger_experiment.dallinger_experiment import extra_files
+                except ImportError:
+                    pass
+
+        if extra_files is not None:
+            for src, filename in extra_files():
+                filename = filename.lstrip("/")
+                if os.path.isdir(src):
+                    for dirpath, dirnames, filenames in os.walk(src, topdown=True):
+                        for fn in filenames:
+                            dst_fileparts = [dst, filename] + dirnames + [fn]
+                            dst_filepath = os.path.join(*dst_fileparts)
+                            yield (
+                                os.path.join(dirpath, fn),
+                                dst_filepath,
+                            )
+                else:
+                    dst_filepath = os.path.join(dst, filename)
+                    yield (src, dst_filepath)
+
+    @property
+    def files(self):
+        """A Set of all files copyable in the source directory, accounting for
+        exclusions.
+        """
+        return {src for (src, dst) in self._get_mapping("")}
+
+    @property
+    def size(self):
+        """Combined size of all files, accounting for exclusions.
+        """
+        return sum([os.path.getsize(path) for path in self.files])
+
+    def selective_copy_to(self, destination):
+        """Write files from the source directory to another directory, skipping
+        files excluded by the general exclusion_policy, plus any files
+        ignored by git configuration.
+        """
+        for from_path, to_path in self._get_mapping(destination):
+            target_folder = os.path.dirname(to_path)
+            ensure_directory(target_folder)
+            shutil.copyfile(from_path, to_path)
+
+
 def assemble_experiment_temp_dir(config):
     """Create a temp directory from which to run an experiment.
     The new directory will include:
@@ -174,8 +246,8 @@ def assemble_experiment_temp_dir(config):
 
     Returns the absolute path of the new directory.
     """
-    app_id = config.get("id")
-    dst = os.path.join(tempfile.mkdtemp(), app_id)
+    exp_id = config.get("id")
+    dst = os.path.join(tempfile.mkdtemp(), exp_id)
 
     # Copy local experiment files, minus some
     ExperimentFileSource(os.getcwd()).selective_copy_to(dst)
@@ -185,15 +257,22 @@ def assemble_experiment_temp_dir(config):
 
     # Save the experiment id
     with open(os.path.join(dst, "experiment_id.txt"), "w") as file:
-        file.write(app_id)
+        file.write(exp_id)
 
     # Copy Dallinger files
     dallinger_root = dallinger_package_path()
     ensure_directory(os.path.join(dst, "static", "scripts"))
     ensure_directory(os.path.join(dst, "static", "css"))
     frontend_files = [
+        os.path.join("static", "css", "bootstrap.min.css"),
         os.path.join("static", "css", "dallinger.css"),
+        os.path.join("static", "css", "dashboard.css"),
+        os.path.join("static", "scripts", "jquery-3.5.1.min.js"),
+        os.path.join("static", "scripts", "popper.min.js"),
+        os.path.join("static", "scripts", "bootstrap.min.js"),
+        os.path.join("static", "scripts", "clipboard.min.js"),
         os.path.join("static", "scripts", "dallinger2.js"),
+        os.path.join("static", "scripts", "network-monitor.js"),
         os.path.join("static", "scripts", "reqwest.min.js"),
         os.path.join("static", "scripts", "require.js"),
         os.path.join("static", "scripts", "reconnecting-websocket.js"),
@@ -207,6 +286,13 @@ def assemble_experiment_temp_dir(config):
         os.path.join("templates", "questionnaire.html"),
         os.path.join("templates", "thanks.html"),
         os.path.join("templates", "waiting.html"),
+        os.path.join("templates", "login.html"),
+        os.path.join("templates", "dashboard_lifecycle.html"),
+        os.path.join("templates", "dashboard_database.html"),
+        os.path.join("templates", "dashboard_heroku.html"),
+        os.path.join("templates", "dashboard_home.html"),
+        os.path.join("templates", "dashboard_monitor.html"),
+        os.path.join("templates", "dashboard_mturk.html"),
         os.path.join("static", "robots.txt"),
     ]
     frontend_dirs = [os.path.join("templates", "base")]
@@ -237,6 +323,8 @@ def assemble_experiment_temp_dir(config):
         src = os.path.join(dallinger_root, "heroku", "Procfile_no_clock")
         shutil.copy(src, os.path.join(dst, "Procfile"))
 
+    ExplicitFileSource(os.getcwd()).selective_copy_to(dst)
+
     return dst
 
 
@@ -266,13 +354,8 @@ def setup_experiment(log, debug=True, verbose=False, app=None, exp_config=None):
     # Generate a unique id for this experiment.
     from dallinger.experiment import Experiment
 
-    generated_uid = public_id = Experiment.make_uuid(app)
-
-    # If the user provided an app name, use it everywhere that's user-facing.
-    if app:
-        public_id = str(app)
-
-    log("Experiment id is " + public_id + "")
+    experiment_uid = heroku_app_id = Experiment.make_uuid(app)
+    log("Experiment UID: {}".format(experiment_uid))
 
     # Load and update the config
     config = get_config()
@@ -280,7 +363,23 @@ def setup_experiment(log, debug=True, verbose=False, app=None, exp_config=None):
         config.load()  #
     if exp_config:
         config.extend(exp_config)
-    config.extend({"id": six.text_type(generated_uid)})
+
+    # If the user provided an app name, store it. We'll use it as the basis for
+    # the Heroku app ID. We still have a fair amount of ambiguity around what
+    # this value actually represents (it's not used as _only_ the Heroku app ID).
+    if app:
+        heroku_app_id = str(app)
+        log("Using custom Heroku ID root: {}".format(heroku_app_id))
+
+    config.extend(
+        {
+            "id": six.text_type(experiment_uid),
+            "heroku_app_id_root": six.text_type(heroku_app_id),
+        }
+    )
+
+    if not config.get("dashboard_password", None):
+        config.set("dashboard_password", fake.password(length=20, special_chars=False))
 
     temp_dir = assemble_experiment_temp_dir(config)
     log("Deployment temp directory: {}".format(temp_dir), chevrons=False)
@@ -289,10 +388,12 @@ def setup_experiment(log, debug=True, verbose=False, app=None, exp_config=None):
     if not debug:
         log("Freezing the experiment package...")
         shutil.make_archive(
-            os.path.join(os.getcwd(), "snapshots", public_id + "-code"), "zip", temp_dir
+            os.path.join(os.getcwd(), "snapshots", heroku_app_id + "-code"),
+            "zip",
+            temp_dir,
         )
 
-    return (public_id, temp_dir)
+    return (heroku_app_id, temp_dir)
 
 
 INITIAL_DELAY = 5
@@ -350,13 +451,14 @@ def deploy_sandbox_shared_setup(log, verbose=True, app=None, exp_config=None):
     if not config.ready:
         config.load()
     heroku.sanity_check(config)
-
-    (id, tmp) = setup_experiment(log, debug=False, app=app, exp_config=exp_config)
+    (heroku_app_id, tmp) = setup_experiment(
+        log, debug=False, app=app, exp_config=exp_config
+    )
 
     # Register the experiment using all configured registration services.
     if config.get("mode") == "live":
         log("Registering the experiment on configured services...")
-        registration.register(id, snapshot=None)
+        registration.register(heroku_app_id, snapshot=None)
 
     # Log in to Heroku if we aren't already.
     log("Making sure that you are logged in to Heroku.")
@@ -372,12 +474,12 @@ def deploy_sandbox_shared_setup(log, verbose=True, app=None, exp_config=None):
     git = GitClient(output=out)
     git.init()
     git.add("--all")
-    git.commit('"Experiment {}"'.format(id))
+    git.commit('"Experiment {}"'.format(heroku_app_id))
 
     # Initialize the app on Heroku.
     log("Initializing app on Heroku...")
     team = config.get("heroku_team", None)
-    heroku_app = HerokuApp(dallinger_uid=id, output=out, team=team)
+    heroku_app = HerokuApp(dallinger_uid=heroku_app_id, output=out, team=team)
     heroku_app.bootstrap()
     heroku_app.buildpack("https://github.com/stomita/heroku-buildpack-phantomjs")
 
@@ -403,8 +505,16 @@ def deploy_sandbox_shared_setup(log, verbose=True, app=None, exp_config=None):
         "smtp_username": config["smtp_username"],
         "smtp_password": config["smtp_password"],
         "whimsical": config["whimsical"],
+        "FLASK_SECRET_KEY": codecs.encode(os.urandom(16), "hex").decode("ascii"),
     }
 
+    # Set up the preferred class as an environment variable, if one is set
+    # This is needed before the config is parsed, but we also store it in the
+    # config to make things easier for recording into bundles.
+    preferred_class = config.get("EXPERIMENT_CLASS_NAME", None)
+    if preferred_class:
+        heroku_config["EXPERIMENT_CLASS_NAME"] = preferred_class
+
     heroku_app.set_multiple(**heroku_config)
 
     # Wait for Redis database to be ready.
@@ -426,6 +536,18 @@ def deploy_sandbox_shared_setup(log, verbose=True, app=None, exp_config=None):
     git.commit("Save URL for database")
     time.sleep(0.25)
 
+    log("Generating dashboard links...")
+    heroku_addons = heroku_app.addon_parameters()
+    heroku_addons = json.dumps(heroku_addons)
+    if six.PY2:
+        heroku_addons = heroku_addons.decode("utf-8")
+    config.extend({"infrastructure_debug_details": heroku_addons})
+    config.write()
+    git.add("config.txt")
+    time.sleep(0.25)
+    git.commit("Save URLs for heroku addon management")
+    time.sleep(0.25)
+
     # Launch the Heroku app.
     log("Pushing code to Heroku...")
     git.push(remote="heroku", branch="HEAD:master")
@@ -449,17 +571,29 @@ def deploy_sandbox_shared_setup(log, verbose=True, app=None, exp_config=None):
     result = {
         "app_name": heroku_app.name,
         "app_home": heroku_app.url,
+        "dashboard_url": "{}/dashboard/".format(heroku_app.url),
         "recruitment_msg": launch_data.get("recruitment_msg", None),
     }
     log("Experiment details:")
     log("App home: {}".format(result["app_home"]), chevrons=False)
+    log("Dashboard URL: {}".format(result["dashboard_url"]), chevrons=False)
+    log("Dashboard user: {}".format(config.get("dashboard_user")), chevrons=False)
+    log(
+        "Dashboard password: {}".format(config.get("dashboard_password")),
+        chevrons=False,
+    )
+
     log("Recruiter info:")
     log(result["recruitment_msg"], chevrons=False)
 
     # Return to the branch whence we came.
     os.chdir(cwd)
 
-    log("Completed deployment of experiment " + id + ".")
+    log(
+        "Completed Heroku deployment of experiment ID {} using app ID {}.".format(
+            config.get("id"), heroku_app_id
+        )
+    )
     return result
 
 
@@ -480,6 +614,7 @@ class HerokuLocalDeployment(object):
     exp_id = None
     tmp_dir = None
     dispatch = {}  # Subclass may provide handlers for Heroku process output
+    environ = None
 
     def configure(self):
         self.exp_config.update({"mode": "debug", "loglevel": 0})
@@ -497,7 +632,7 @@ class HerokuLocalDeployment(object):
         if logfile and logfile != "-":
             logfile = os.path.join(self.original_dir, logfile)
             config.extend({"logfile": logfile})
-            config.write()
+        config.write()
 
     def run(self):
         """Set up the environment, get a HerokuLocalWrapper instance, and pass
@@ -509,7 +644,13 @@ class HerokuLocalDeployment(object):
         db.init_db(drop_all=True)
         self.out.log("Starting up the server...")
         config = get_config()
-        with HerokuLocalWrapper(config, self.out, verbose=self.verbose) as wrapper:
+        environ = None
+        if self.environ:
+            environ = os.environ.copy()
+            environ.update(self.environ)
+        with HerokuLocalWrapper(
+            config, self.out, verbose=self.verbose, env=environ
+        ) as wrapper:
             try:
                 self.execute(wrapper)
             except KeyboardInterrupt:
@@ -550,6 +691,9 @@ class DebugDeployment(HerokuLocalDeployment):
         self.complete = False
         self.status_thread = None
         self.no_browsers = no_browsers
+        self.environ = {
+            "FLASK_SECRET_KEY": codecs.encode(os.urandom(16), "hex").decode("ascii"),
+        }
 
     def configure(self):
         super(DebugDeployment, self).configure()
@@ -572,6 +716,10 @@ class DebugDeployment(HerokuLocalDeployment):
         else:
             if result["status"] == "success":
                 self.out.log(result["recruitment_msg"])
+                dashboard_url = "{}/dashboard/".format(get_base_url())
+                self.display_dashboard_access_details(dashboard_url)
+                if not self.no_browsers:
+                    self.open_dashboard(dashboard_url)
                 self.heroku = heroku
                 heroku.monitor(listener=self.notify)
 
@@ -597,6 +745,24 @@ class DebugDeployment(HerokuLocalDeployment):
             url = url.replace(str(get_config().get("base_port")), self.proxy_port)
         new_webbrowser_profile().open(url, new=1, autoraise=True)
 
+    def display_dashboard_access_details(self, url):
+        config = get_config()
+        self.out.log("Experiment dashboard: {}".format(url))
+        self.out.log(
+            "Dashboard user: {} password: {}".format(
+                config.get("dashboard_user"), config.get("dashboard_password"),
+            )
+        )
+
+    def open_dashboard(self, url):
+        config = get_config()
+        self.out.log("Opening dashboard")
+        parsed = list(urlparse(url))
+        parsed[1] = "{}:{}@{}".format(
+            config.get("dashboard_user"), config.get("dashboard_password"), parsed[1],
+        )
+        new_webbrowser_profile().open(urlunparse(parsed), new=1, autoraise=True)
+
     def recruitment_closed(self, match):
         """Recruitment is closed.
 
