@@ -5,6 +5,7 @@ from __future__ import unicode_literals
 
 from cached_property import cached_property
 from collections import Counter
+from collections import OrderedDict
 from contextlib import contextmanager
 from functools import wraps
 import datetime
@@ -18,9 +19,11 @@ import requests
 import sys
 import time
 import uuid
+import warnings
 
 from sqlalchemy import and_
 from sqlalchemy import create_engine
+from sqlalchemy import distinct
 from sqlalchemy import func
 from sqlalchemy.orm import sessionmaker, scoped_session
 
@@ -34,12 +37,14 @@ from dallinger.data import load as data_load
 from dallinger.data import find_experiment_export
 from dallinger.data import ingest_zip
 from dallinger.db import init_db, db_url
-from dallinger.models import Network, Node, Info, Transformation, Participant
+from dallinger import models
+from dallinger.models import Network, Node, Info, Transformation, Participant, Vector
 from dallinger.heroku.tools import HerokuApp
 from dallinger.information import Gene, Meme, State
 from dallinger.nodes import Agent, Source, Environment
 from dallinger.transformations import Compression, Response
 from dallinger.transformations import Mutation, Replication
+from dallinger.utils import struct_to_html
 
 
 from dallinger.networks import Empty
@@ -664,6 +669,207 @@ class Experiment(object):
         """
         return None
 
+    def monitoring_panels(self, **kw):
+        """Provides monitoring dashboard sidebar panels.
+
+        :param \**kw: arguments passed in from the request
+        :returns: An ``OrderedDict()`` mapping panel titles to HTML strings
+                  to render in the dashboard sidebar.
+        """  # noqa
+        stats = self.monitoring_statistics(**kw)
+        panels = OrderedDict()
+        for tab in stats:
+            panels[tab] = struct_to_html(stats[tab])
+        return panels
+
+    def monitoring_statistics(self, **kw):
+        """The default data used for the monitoring panels
+
+        :param \**kw: arguments passed in from the request
+        :returns: An ``OrderedDict()`` mapping panel titles to data structures
+                  describing the experiment state.
+        """  # noqa
+        participants = Participant.query
+        nodes = Node.query
+        infos = Info.query
+
+        stats = OrderedDict()
+        stats["Participants"] = OrderedDict(
+            (
+                ("working", participants.filter_by(status="working").count()),
+                ("abandoned", participants.filter_by(status="abandoned").count()),
+                ("returned", participants.filter_by(status="returned").count()),
+                ("approved", participants.filter_by(status="approved").count()),
+            )
+        )
+
+        # Active participants have more than 10 infos
+        active_participants = 0
+        for p in participants:
+            if len(p.infos(failed="all")) > 10:
+                active_participants += 1
+        stats["Participants"]["active"] = active_participants
+
+        # Count up our networks by role
+        network_roles = self.session.query(Network.role, func.count(Network.role))
+        network_counts = network_roles.group_by(Network.role).all()
+        failed_networks = network_roles.filter(Network.failed == True)  # noqa
+        failed_counts = dict(failed_networks.group_by(Network.role).all())
+        network_stats = {}
+        for role, count in network_counts:
+            network_stats[role] = OrderedDict(
+                (("count", count), ("failed", failed_counts.get(role, 0)),)
+            )
+        stats["Networks"] = network_stats
+
+        stats["Nodes"] = OrderedDict(
+            (
+                ("count", nodes.count()),
+                ("failed", nodes.filter_by(failed=True).count()),
+            )
+        )
+
+        stats["Infos"] = OrderedDict(
+            (
+                ("count", infos.count()),
+                ("failed", infos.filter_by(failed=True).count()),
+            )
+        )
+
+        if kw.get("transformations"):
+            transformations = Transformation.query
+            stats["transformations"] = OrderedDict(
+                (
+                    ("count", transformations.count()),
+                    ("failed", transformations.filter_by(failed=True).count()),
+                )
+            )
+
+        return stats
+
+    def network_structure(self, **kw):
+        network_ids = {i[0] for i in self.session.query(distinct(Network.id)).all()}
+        if "network_roles" in kw:
+            network_ids = {
+                i[0]
+                for i in self.session.query(distinct(Network.id)).filter(
+                    Network.role.in_(kw["network_roles"])
+                )
+            }
+        if "network_ids" in kw:
+            network_ids = network_ids.intersection(int(v) for v in kw["network_ids"])
+
+        jnetworks = [
+            n.__json__()
+            for n in Network.query.filter(Network.id.in_(network_ids)).all()
+        ]
+        if "collapsed" in kw:
+            # Collapsed view shows Source nodes only
+            jnodes = [
+                n.__json__()
+                for n in Source.query.filter(Node.network_id.in_(network_ids)).all()
+            ]
+            jinfos = jparticipants = jtransformations = jvectors = []
+        else:
+            jnodes = [
+                n.__json__()
+                for n in Node.query.filter(Node.network_id.in_(network_ids)).all()
+            ]
+            jinfos = [
+                n.__json__()
+                for n in Info.query.filter(Info.network_id.in_(network_ids)).all()
+            ]
+            # We don't filter participants because they aren't directly connected to specific networks
+            jparticipants = [n.__json__() for n in Participant.query.all()]
+            jtransformations = []
+            if kw.get("transformations"):
+                jtransformations = [
+                    n.__json__()
+                    for n in Transformation.query.filter(
+                        Transformation.network_id.in_(network_ids)
+                    ).all()
+                ]
+
+            jvectors = [
+                {
+                    "origin_id": v.origin_id,
+                    "destination_id": v.destination_id,
+                    "id": v.id,
+                    "failed": v.failed,
+                }
+                for v in Vector.query.filter(Vector.network_id.in_(network_ids)).all()
+            ]
+
+        return {
+            "networks": jnetworks,
+            "nodes": jnodes,
+            "vectors": jvectors,
+            "infos": jinfos,
+            "participants": jparticipants,
+            "trans": jtransformations,
+        }
+
+    def node_visualization_html(self, object_type, obj_id):
+        """Returns a string with custom HTML visualization for a given object
+        referenced by the object base type and id.
+
+        :param object_type: The base object class name, e.g. ``Network``, ``Node``, ``Info``, ``Participant``, etc.
+        :type object_type: str
+        :param id: The ``id`` of the object
+        :type id: int
+
+        :returns: A valid HTML string to be inserted into the monitoring dashboard
+        """
+
+        model = getattr(models, object_type, None)
+        if model is not None:
+            obj = self.session.query(model).get(int(obj_id))
+            if getattr(obj, "visualization_html", None):
+                return obj.visualization_html
+        return ""
+
+    def table_data(self, **kw):
+        """Generates DataTablesJS data and configuration for the experiment. The data
+        is compiled from the models' ``__json__`` methods, and can be customized by either
+        overriding this method or using the ``json_data`` method on the model to return
+        additional serializable data.
+
+        :param \**kw: arguments passed in from the request. The ``model_type`` parameter
+                      takes a ``str`` or iterable and queries all objects of those types,
+                      ordered by ``id``.
+        :returns: Returns a a ``dict`` with DataTablesJS data and configuration, filters using
+                  arbitrary keyword arguments. Should contain ``data`` and ``columns`` keys
+                  at least, with ``columns`` containing data for all fields on all returned
+                  objects.
+        """  # noqa
+        rows = []
+        found_columns = set()
+        columns = []
+        model_types = kw.get("model_type", ["Participant"])
+        if hasattr(model_types, "strip"):
+            model_types = [model_types]
+
+        for model_type in model_types:
+            model = getattr(models, model_type, None)
+            for obj in model.query.order_by(model.id).all():
+                data = obj.__json__()
+                rows.append(data)
+                for key in data:
+                    if key not in found_columns:
+                        columns.append({"name": key, "data": key})
+                        found_columns.add(key)
+
+        # Make sure every row has an entry for every column
+        for col in found_columns:
+            for row in rows:
+                if col not in row:
+                    row[col] = None
+
+        return {
+            "data": rows,
+            "columns": columns,
+        }
+
     @property
     def usable_replay_range(self):
         """The range of times that represent the active part of the experiment"""
@@ -936,6 +1142,12 @@ class Scrubber(object):
         display(self.widget())
 
 
+def is_experiment_class(cls):
+    return (
+        inspect.isclass(cls) and issubclass(cls, Experiment) and cls is not Experiment
+    )
+
+
 def load():
     """Load the active experiment."""
     initialize_experiment_package(os.getcwd())
@@ -945,12 +1157,38 @@ def load():
         except ImportError:
             from dallinger_experiment import dallinger_experiment as experiment
 
-        classes = inspect.getmembers(experiment, inspect.isclass)
-        for name, c in classes:
-            if "Experiment" in c.__bases__[0].__name__:
-                return c
+        classes = inspect.getmembers(experiment, is_experiment_class)
+
+        preferred_class = os.environ.get("EXPERIMENT_CLASS_NAME", None)
+        if preferred_class is not None:
+            try:
+                return dict(classes)[preferred_class]
+            except KeyError:
+                raise ImportError(
+                    "No experiment named {} was found".format(preferred_class)
+                )
+
+        if len(classes) > 1:
+            for name, c in classes:
+                if "Experiment" in c.__bases__[0].__name__:
+                    warnings.warn(
+                        UserWarning(
+                            "More than one potential experiment class found but no EXPERIMENT_CLASS_NAME environment variable. Picking {} from {}.".format(
+                                name, [n for (n, cls) in classes]
+                            )
+                        ),
+                        stacklevel=3,
+                    )
+                    return c
+            raise ImportError(
+                "No direct experiment subclass found in {}".format(
+                    [n for (n, cls) in classes]
+                )
+            )
+        elif len(classes) == 0:
+            raise ImportError("No experiment classes found")
         else:
-            raise ImportError
+            return classes[0][1]
     except ImportError:
         logger.error("Could not import experiment.")
         raise
