@@ -7,7 +7,16 @@ from json import loads
 import os
 import re
 
-from flask import abort, Flask, render_template, request, Response, send_from_directory
+from flask import (
+    abort,
+    Flask,
+    render_template,
+    request,
+    Response,
+    send_from_directory,
+    url_for,
+)
+from flask_login import LoginManager
 from jinja2 import TemplateNotFound
 from rq import Queue
 from sqlalchemy.orm.exc import NoResultFound, MultipleResultsFound
@@ -24,6 +33,7 @@ from dallinger import recruiters
 from dallinger.notifications import admin_notifier
 from dallinger.notifications import MessengerError
 
+from . import dashboard
 from .replay import ReplayBackend
 from .worker_events import worker_function
 from .utils import (
@@ -50,9 +60,15 @@ app = Flask("Experiment_Server")
 
 @app.before_first_request
 def _config():
+    app.secret_key = app.config["SECRET_KEY"] = os.environ.get("FLASK_SECRET_KEY")
     config = get_config()
     if not config.ready:
         config.load()
+    if config.get("dashboard_password", None):
+        app.config["ADMIN_USER"] = dashboard.User(
+            userid=config.get("dashboard_user", "admin"),
+            password=config.get("dashboard_password"),
+        )
 
     return config
 
@@ -75,19 +91,28 @@ else:
 # primary recruiter's route:
 app.register_blueprint(recruiters.mturk_routes)
 
+# Load dashboard routes and login setup
+app.register_blueprint(dashboard.dashboard)
+login = LoginManager(app)
+login.login_view = "dashboard.login"
+login.request_loader(dashboard.load_user_from_request)
+login.user_loader(dashboard.load_user)
+login.unauthorized_handler(dashboard.unauthorized)
+app.config["dashboard_tabs"] = dashboard.dashboard_tabs
+
 """Basic routes."""
 
 
 @app.route("/")
 def index():
     """Index route"""
-    config = _config()
-    html = "<html><head></head><body><h1>Dallinger Experiment in progress</h1><dl>"
-    for item in sorted(config.as_dict().items()):
-        html += '<dt style="font-weight:bold;margin-top:15px;">{}</dt><dd>{}</dd>'.format(
-            *item
+    html = (
+        "<html><head></head><body><h1>Dallinger Experiment in progress</h1>"
+        "<p><a href={}>Dashboard</a></p></body></html>".format(
+            url_for("dashboard.index")
         )
-    html += "</dl></body></html>"
+    )
+
     return html
 
 
