@@ -259,6 +259,10 @@ class MTurkService(object):
             template = u"https://mturk-requester.{}.amazonaws.com"
         return template.format(self.region_name)
 
+    def account_balance(self):
+        response = self.mturk.get_account_balance()
+        return float(response["AvailableBalance"])
+
     def check_credentials(self):
         """Verifies key/secret/host combination by making a balance inquiry"""
         try:
@@ -502,7 +506,7 @@ class MTurkService(object):
             )
         except Exception as ex:
             raise MTurkServiceException(
-                "Failed to extend time until expiration of HIT: {}".format(
+                "Failed to extend time until expiration of HIT: {}\n{}".format(
                     expiration, str(ex)
                 )
             )
@@ -697,6 +701,9 @@ class MTurkService(object):
             "status": hit["HITStatus"],
             "annotation": hit.get("RequesterAnnotation"),
             "worker_url": self._worker_hit_url(hit["HITTypeId"]),
+            "assignments_available": hit["NumberOfAssignmentsAvailable"],
+            "assignments_completed": hit["NumberOfAssignmentsCompleted"],
+            "assignments_pending": hit["NumberOfAssignmentsPending"],
         }
 
         return translated
