@@ -6,6 +6,7 @@ import shutil
 import sys
 import tempfile
 import time
+from jinja2 import FileSystemLoader
 from selenium import webdriver
 from dallinger import information
 from dallinger import models
@@ -47,9 +48,15 @@ def cwd(root):
     os.chdir(root)
 
 
-@pytest.fixture
+def is_dallinger_module(key):
+    return key.startswith("dallinger_experiment") or key.startswith(
+        "TEST_EXPERIMENT_UID"
+    )
+
+
+@pytest.fixture(autouse=True)
 def reset_sys_modules():
-    to_clear = [k for k in sys.modules if k.startswith("dallinger_experiment")]
+    to_clear = [k for k in sys.modules if is_dallinger_module(k)]
     for key in to_clear:
         del sys.modules[key]
 
@@ -77,19 +84,17 @@ def env():
     # Heroku requires a home directory to start up
     # We create a fake one using tempfile and set it into the
     # environment to handle sandboxes on CI servers
-    environ_orig = os.environ.copy()
-    running_on_ci = environ_orig.get("CI", False)
-    have_home_dir = environ_orig.get("HOME", False)
-    if not running_on_ci and have_home_dir:
-        yield environ_orig
-    else:
-        fake_home = tempfile.mkdtemp()
-        environ_patched = environ_orig.copy()
-        environ_patched.update({"HOME": fake_home})
-        os.environ = environ_patched
-        yield environ_patched
-        os.environ = environ_orig
-        shutil.rmtree(fake_home, ignore_errors=True)
+    with mock.patch("os.environ", os.environ.copy()) as environ_patched:
+        running_on_ci = environ_patched.get("CI", False)
+        have_home_dir = environ_patched.get("HOME", False)
+        environ_patched.update({"FLASK_SECRET_KEY": "A TERRIBLE SECRET"})
+        if not running_on_ci and have_home_dir:
+            yield environ_patched
+        else:
+            fake_home = tempfile.mkdtemp()
+            environ_patched.update({"HOME": fake_home})
+            yield environ_patched
+            shutil.rmtree(fake_home, ignore_errors=True)
 
 
 @pytest.fixture
@@ -131,15 +136,17 @@ def stub_config():
         u"dallinger_email_address": u"test@example.com",
         u"database_size": u"standard-0",
         u"redis_size": u"premium-0",
+        u"dashboard_user": u"admin",
         u"database_url": u"postgresql://postgres@localhost/dallinger",
         u"description": u"fake HIT description",
         u"duration": 1.0,
         u"dyno_type": u"free",
+        u"heroku_app_id_root": u"fake-customid",
         u"heroku_auth_token": u"heroku secret",
         u"heroku_python_version": u"3.6.10",
         u"heroku_team": u"",
         u"host": u"0.0.0.0",
-        u"id": u"some experiment uid",
+        u"id": u"TEST_EXPERIMENT_UID",  # This is a significant value; change with caution.
         u"keywords": u"kw1, kw2, kw3",
         u"lifetime": 1,
         u"logfile": u"-",
@@ -187,6 +194,14 @@ def active_config(stub_config):
     c.config = orig_config
 
 
+@pytest.fixture
+def dashboard_config(active_config):
+    active_config.extend(
+        {"dashboard_user": u"admin", "dashboard_password": u"DUMBPASSWORD"}
+    )
+    return active_config
+
+
 @pytest.fixture
 def db_session():
     import dallinger.db
@@ -314,6 +329,11 @@ def a(db_session):
             # nodes.Source is intended to be abstract
             return self._build(nodes.RandomBinaryStringSource, defaults)
 
+        def transformation(self, **kw):
+            defaults = {}
+            defaults.update(kw)
+            return self._build(models.Transformation, defaults)
+
         def _build(self, klass, attrs):
             # Some of our default values are factories:
             for k, v in attrs.items():
@@ -331,15 +351,37 @@ def a(db_session):
     return ModelFactory(db_session)
 
 
+def uncached_jinja_loader(app):
+    """We want a non-cached template loader so we can load templates from
+    directories which may vary between tests, so override
+    the @locked_cached_property from flask.helpers
+    """
+    if app.template_folder is not None:
+        return FileSystemLoader(os.path.join(app.root_path, app.template_folder))
+
+
 @pytest.fixture
-def webapp(active_config):
+def webapp(active_config, reset_sys_modules, env):
+    """Return a Flask test client.
+
+    The imported app assumes an active Configuration, and will load both the
+    test experiment package and templates from the current directory,
+    so we need to make sure we wipe the slate clean between tests. This means
+    not caching the Flask template search path, and clearing out sys.modules
+    before loading the Flask app.
+    """
     from dallinger.experiment_server import sockets
 
     app = sockets.app
-    app.root_path = os.getcwd()  # look in the right place for test's templates
+    # look in the cwd for test's templates, and make sure the template loader
+    # uses that directory to search for them.
+    app.root_path = os.getcwd()
+    app.jinja_loader = uncached_jinja_loader(app)
+
     app.config.update({"DEBUG": True, "TESTING": True})
     client = app.test_client()
     yield client
+    app._got_first_request = False
 
 
 @pytest.fixture
