@@ -475,7 +475,7 @@ class MTurkRecruiter(Recruiter):
         the Mechanical Turk site to submit their HIT, which in turn triggers
         notifications to the /notifications route.
         """
-        if self.config.get("mode") == "sandbox":
+        if self.is_sandbox:
             return "https://workersandbox.mturk.com/mturk/externalSubmit"
         return "https://www.mturk.com/mturk/externalSubmit"
 
@@ -549,7 +549,7 @@ class MTurkRecruiter(Recruiter):
             "reward": float(dollars),
             "duration_hours": 1,
             "lifetime_days": 3,
-            "question": MTurkQuestions.compensation(sandbox=self._is_sandbox),
+            "question": MTurkQuestions.compensation(sandbox=self.is_sandbox),
             "qualifications": [MTurkQualificationRequirements.must_have(qid)],
             "do_subscribe": False,
         }
@@ -722,7 +722,7 @@ class MTurkRecruiter(Recruiter):
         #     logger.exception(str(ex))
 
     @property
-    def _is_sandbox(self):
+    def is_sandbox(self):
         return self.config.get("mode") == "sandbox"
 
     def _build_hit_qualifications(self):
@@ -732,10 +732,15 @@ class MTurkRecruiter(Recruiter):
             quals.append(reqs.min_approval(self.config.get("approve_requirement")))
         if self.config.get("us_only"):
             quals.append(reqs.restrict_to_countries(["US"]))
-        for item in self._config_to_list("qualification_blacklist"):
+        for item in self._config_to_list("mturk_qualification_blocklist"):
             qtype = self.mturkservice.get_qualification_type_by_name(item)
             if qtype:
                 quals.append(reqs.must_not_have(qtype["id"]))
+        if self.config.get("mturk_qualification_requirements", None) is not None:
+            explicit_qualifications = json.loads(
+                self.config.get("mturk_qualification_requirements")
+            )
+            quals.extend(explicit_qualifications)
 
         return quals
 
@@ -759,7 +764,7 @@ class MTurkRecruiter(Recruiter):
         return status
 
     def _disable_autorecruit(self):
-        heroku_app = heroku_tools.HerokuApp(self.config.get("id"))
+        heroku_app = heroku_tools.HerokuApp(self.config.get("heroku_app_id_root"))
         args = json.dumps({"auto_recruit": "false"})
         headers = heroku_tools.request_headers(self.config.get("heroku_auth_token"))
         requests.patch(heroku_app.config_url, data=args, headers=headers)
